SuperStrict

' Holzchopf - Merkli
' miniBCC #29

Framework maxgui.drivers
Import brl.eventqueue
Import brl.random

' game consts
Const CARD_SIZE:Int		= 58
Const CARD_FONTSIZE:Int	= 36
Const DECK_MINSIZE:Int	= 4
Const DECK_MAXSIZE:Int	= 10
Const INFO_HEIGHT:Int	= 24
Const INFO_FONTSIZE:Int	= 14

Const SYMBOLS_LOWER:Int	= 9728
Const SYMBOLS_UPPER:Int	= 9838
Global SYMBOLS_LOCKED:Int[]	= [9748, 9749, 9752, 9753]

' app consts
Const MTAG_SIZE:Int		= 100
Const MTAG_PLAYERS:Int	= 200
Const MTAG_RESTART:Int	= 300
Const MTAG_QUIT:Int		= 400

Global BG_RED:Byte, BG_GRN:Byte, BG_BLU:Byte
Global FG_RED:Byte, FG_GRN:Byte, FG_BLU:Byte
Global BGI_RED:Byte, BGI_GRN:Byte, BGI_BLU:Byte
Global FGI_RED:Byte, FGI_GRN:Byte, FGI_BLU:Byte
Global DIS_RED:Byte, DIS_GRN:Byte, DIS_BLU:Byte
Global CARDBACK_RED:Byte, CARDBACK_GRN:Byte, CARDBACK_BLU:Byte
Global CARDSYM_RED:Byte, CARDSYM_GRN:Byte, CARDSYM_BLU:Byte

' default game settings
Global DECK_WIDTH:Int	= 5
Global DECK_HEIGHT:Int	= 4
Global PLAYERS:Int		= 1

Global window:TGadget
Global infofields:TGadget[4]
Global menu:TGadget
Global menu_game:TGadget
Global menu_skill:TGadget
Global menu_players:TGadget

' set fonts and colors
Global infofont:TGuiFont
infofont = LookupGuiFont(GUIFONT_SANSSERIF, INFO_FONTSIZE, FONT_BOLD)
Global cardfont:TGuiFont
cardfont = LookupGuiFont(GUIFONT_SANSSERIF, CARD_FONTSIZE)
LookupGuiColor(GUICOLOR_WINDOWBG, BG_RED, BG_GRN, BG_BLU)
LookupGuiColor(GUICOLOR_GADGETFG, FG_RED, FG_GRN, FG_BLU)
BGI_RED		= 255 -BG_RED
BGI_GRN		= 255 -BG_GRN
BGI_BLU		= 255 -BG_BLU
FGI_RED		= 255 -FG_RED
FGI_GRN		= 255 -FG_GRN
FGI_BLU		= 255 -FG_BLU
DIS_RED		= 0.9 * BG_RED +0.1 * FG_RED
DIS_GRN		= 0.9 * BG_GRN +0.1 * FG_GRN
DIS_BLU		= 0.9 * BG_BLU +0.1 * FG_BLU
CARDBACK_RED= 128
CARDBACK_GRN= 128
CARDBACK_BLU= 140
CARDSYM_RED	= 0
CARDSYM_GRN	= 0
CARDSYM_BLU	= 0

' app initialisation
SeedRnd(MilliSecs())
InitWindow()

Global player:Int = 0
Global points:Int[4]
Global picks:TCard[2]
Global picks_cnt:Int
Global cards_total:Int
Global cards_remaining:Int

' symbols to appear under cards
Type TSymbol
	Global list:TSymbol[]
	Field code:Int
	Field hue:Int
	
	' create random symbols to use
	Function PrepareSymbols(pCount:Int)
		list = New TSymbol[0]
		Local hstep:Int = 360 / pCount
		Local h:Int = 0, c:Int
		For Local i:Int = 0 Until pCount
			' pick random available symbol
			Repeat
				c = Rand(SYMBOLS_LOWER, SYMBOLS_UPPER)
			Until SymbolIsAvailable(c)
			' create symbol and store ascii-code and hue
			Local symbol:TSymbol = New TSymbol
			symbol.code = c
			symbol.hue	= h
			list :+ [symbol]
			h :+ hstep
		Next
	End Function
	
	' checks if symbol is available or already in use
	Function SymbolIsAvailable:Int(pCode:Int)
		' symbol is locked
		For Local c:Int = EachIn SYMBOLS_LOCKED
			If pCode = c then
				Return False
			EndIf
		Next
		' symbol is already in use
		For Local sym:TSymbol = EachIn list
			If pCode = sym.code then
				Return False
			EndIf
		Next
		' symbol is available
		Return True
	End Function
	
	' write symbol color into pR, pG, pB
	Method WriteColor(pR:Int Var, pG:Int Var, pB:Int Var)
		pR = Min(Max(96, 176 +cos(hue     ) *180), 255)
		pG = Min(Max(96, 176 +cos(hue -120) *180), 255)
		pB = Min(Max(96, 176 +cos(hue -240) *180), 255)
	End Method
End Type

' cards to be played
Type TCard
	Field xpos:Int, ypos:Int
	Field symbol:TSymbol
	Field gadget:TGadget
	Field uncovered:Int = False
	
	' uncover card, show symbol
	Method Uncover()
		Local r:Int, g:Int, b:Int
		symbol.WriteColor(r, g, b)
		gadget.SetText( Chr(symbol.code) )
		gadget.SetTextColor(CARDSYM_RED, CARDSYM_GRN, CARDSYM_BLU)
		gadget.SetColor(r, g, b)
		uncovered = True
	End Method
	' cover card, show back
	Method Cover()
		gadget.SetText( "?" )
		gadget.SetColor(BG_RED, BG_GRN, BG_BLU)
		gadget.SetTextColor(CARDBACK_RED, CARDBACK_GRN, CARDBACK_BLU)
		uncovered = False
	End Method
	
	Method Remove()
		gadget.Free()
	End Method
End Type

' deck of all cards
Type TDeck
	Global cards:TCard[,]
	
	' create nescessary symbols, cards and play them
	Function PrepareDeck()
		cards = New TCard[DECK_WIDTH, DECK_HEIGHT]
		Local ccards:Int = DECK_WIDTH *DECK_HEIGHT
		Local csymbols:Int = ccards / 2
		Local symbolstopick:TList = CreateList()
		cards_total = ccards
		cards_remaining = ccards
		' fill deck with cards
		TSymbol.PrepareSymbols(csymbols)
		For Local i:Int = 0 Until csymbols
			symbolstopick.AddLast(TSymbol.list[i])
			symbolstopick.AddLast(TSymbol.list[i])
		Next
		' play cards
		For Local y:Int = 0 Until DECK_HEIGHT
			For Local x:Int = 0 Until DECK_WIDTH
				' pick random card
				Local symbol:TSymbol = TSymbol(symbolstopick.ValueAtIndex(Rand(0, ccards -1)))
				symbolstopick.Remove(symbol)
				ccards :- 1
				' place card
				Local card:TCard = New TCard
				card.xpos = x
				card.ypos = y
				card.symbol = symbol
				card.gadget = CreateButton("", x * CARD_SIZE, INFO_HEIGHT +y * CARD_SIZE, CARD_SIZE, CARD_SIZE, window)
				If cardfont then card.gadget.SetFont(cardfont)
				card.Cover()
				'card.Uncover()
				card.gadget.extra = card
				cards[x,y] = card
			Next
		Next
	End Function
	
	Function ClearDeck()
		For Local card:TCard = EachIn cards
			card.Remove()
		Next
	End Function
End Type

' initialize all gadgets and get ready to play
Function InitWindow()
	' create window
	If window then
		TDeck.ClearDeck()
		window.Free()
		GCCollect()
	EndIf
	Local flags:Int = WINDOW_CLIENTCOORDS | WINDOW_TITLEBAR | WINDOW_STATUS | WINDOW_MENU | WINDOW_CENTER
	window = CreateWindow("Merkli", 0,0, DECK_WIDTH * CARD_SIZE, DECK_HEIGHT * CARD_SIZE +2 * INFO_HEIGHT,, flags)
	' create menues
	menu = WindowMenu(window)
	menu_game = CreateMenu("&Spiel", 0, menu)
	menu_skill = CreateMenu("&Grsse", MTAG_SIZE, menu_game)
	menu_players = CreateMenu("S&pieler", MTAG_PLAYERS, menu_game)
	CreateMenu("", 0, menu_game)
	CreateMenu("Spiel &neustarten", MTAG_RESTART, menu_game)
	CreateMenu("Spiel &beenden", MTAG_QUIT, menu_game)
	' deck size menu
	For Local h:Int = DECK_MINSIZE To DECK_MAXSIZE
		Local w:Int
		w = h +1
		Local s:Int = w * h / 2
		Local menu:TGadget = CreateMenu(w +" x &" +h +"~t" +s +" Symbole", MTAG_SIZE +h, menu_skill)
		If w = DECK_WIDTH then
			CheckMenu(menu)
		EndIf
	Next
	' player count menu
	For Local i:Int = 1 To 4
		Local menu:TGadget = CreateMenu("&" +i +" Spieler", MTAG_PLAYERS +i, menu_players)
		If i = PLAYERS then
			CheckMenu(menu)
		EndIf
	Next
	UpdateWindowMenu(window)
	' create info labels
	Local x:Int, y:Int, w:Int, h:Int, flag:Int
	w = DECK_WIDTH * CARD_SIZE /2
	h = INFO_HEIGHT
	For Local i:Int = 0 Until 4
		flag = LABEL_LEFT
		If x > 0 then flag = LABEL_RIGHT
		infofields[i] = CreateLabel("", x,y, w,h, window, flag)
		If infofont then infofields[i].SetFont(infofont)
		infofields[i].SetTextColor(DIS_RED, DIS_GRN, DIS_BLU)
		x :+ w
		If x > w then
			x = 0
			y :+ DECK_HEIGHT * CARD_SIZE + INFO_HEIGHT
		EndIf
	Next
	' play cards
	TDeck.PrepareDeck()
	' start game
	player = 0
	points = New Int[4]
	picks = New TCard[2]
	picks_cnt = 0
	ActivateWindow(window)
End Function

' main loop
Repeat
	' event handling
	WaitEvent()
	Select EventID()
	' quit
	Case EVENT_WINDOWCLOSE, EVENT_APPTERMINATE
		End
	Case EVENT_GADGETACTION
		Local card:TCard = TCard(TGadget(EventSource()).extra)
		' check if source is card
		If card then
			' pick card
			If picks_cnt < 2 then
				If Not card.uncovered then
					card.Uncover()
					picks[picks_cnt] = card
					picks_cnt :+ 1
				EndIf
			Else
				' second card picked, check if they are equal
				' same symbol, remove cards and count
				If picks[0].symbol.code = picks[1].symbol.code then
					picks[0].Remove()
					picks[1].Remove()
					cards_remaining :- 2
					points[player] :+ 1
				' not same symbol, cover cards
				Else
					picks[0].Cover()
					picks[1].Cover()
					player = (player +1) Mod PLAYERS
				EndIf
				picks_cnt = 0
			EndIf
		EndIf
	Case EVENT_MENUACTION
		Local data:Int = EventData()
		If data >= MTAG_SIZE And data < MTAG_SIZE+100 then
			DECK_HEIGHT = data -MTAG_SIZE
			DECK_WIDTH = DECK_HEIGHT +1
			InitWindow()
		ElseIf data >= MTAG_PLAYERS And data < MTAG_PLAYERS+100 then
			PLAYERS = data -MTAG_PLAYERS
			InitWindow()
		ElseIf data = MTAG_RESTART then
			InitWindow()
		ElseIf data = MTAG_QUIT then
			End
		EndIf
	End Select
	
	' update status texts
	window.SetStatusText("Spieler " +(player+1) +"~t~tKarten: "+cards_remaining+" / "+cards_total)
	For Local i:Int = 0 Until 4
		infofields[i].SetText("Spieler " +(i+1) +": " +points[i])
		If i = player then
			infofields[i].SetColor(BGI_RED, BGI_GRN, BGI_BLU)
			infofields[i].SetTextColor(FGI_RED, FGI_GRN, FGI_BLU)
		ElseIf i < players then
			infofields[i].SetColor(BG_RED, BG_GRN, BG_BLU)
			infofields[i].SetTextColor(FG_RED, FG_GRN, FG_BLU)
		EndIf
	Next
Forever
